diff --git a/common/cabac.c b/common/cabac.c
index 14db0cd..1f00acb 100644
--- a/common/cabac.c
+++ b/common/cabac.c
@@ -1325,7 +1325,7 @@ const uint16_t x264_cabac_entropy[128] =
     FIX8(0.9285), FIX8(1.0752), FIX8(1.0000), FIX8(1.0000)
 };
 
-uint8_t x264_cabac_contexts[4][QP_MAX_SPEC+1][1024];
+uint8_t x264_cabac_contexts[4][QP_MAX_SPEC+1][1024] X264_LARGE_BSS_LINK_SECTION;
 
 void x264_cabac_init( x264_t *h )
 {
diff --git a/common/common.c b/common/common.c
index 14d4670..e3273c2 100644
--- a/common/common.c
+++ b/common/common.c
@@ -39,7 +39,9 @@ const int x264_bit_depth = BIT_DEPTH;
 
 const int x264_chroma_format = X264_CHROMA_FORMAT;
 
+#if HAVE_FILEIO
 static void x264_log_default( void *, int, const char *, va_list );
+#endif
 
 /****************************************************************************
  * x264_param_default:
@@ -112,10 +114,12 @@ void x264_param_default( x264_param_t *param )
     param->rc.f_aq_strength = 1.0;
     param->rc.i_lookahead = 40;
 
+#if HAVE_FILEIO
     param->rc.b_stat_write = 0;
     param->rc.psz_stat_out = "x264_2pass.log";
     param->rc.b_stat_read = 0;
     param->rc.psz_stat_in = "x264_2pass.log";
+#endif
     param->rc.f_qcompress = 0.6;
     param->rc.f_qblur = 0.5;
     param->rc.f_complexity_blur = 20;
@@ -123,7 +127,9 @@ void x264_param_default( x264_param_t *param )
     param->rc.b_mb_tree = 1;
 
     /* Log */
+#if HAVE_FILEIO
     param->pf_log = x264_log_default;
+#endif
     param->p_log_private = NULL;
     param->i_log_level = X264_LOG_INFO;
 
@@ -591,7 +597,7 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
     if( strchr( name, '_' ) ) // s/_/-/g
     {
         char *c;
-        name_buf = strdup(name);
+        name_buf = x264_strdup(name);
         if( !name_buf )
             return X264_PARAM_BAD_NAME;
         while( (c = strchr( name_buf, '_' )) )
@@ -617,7 +623,7 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
                  !strcasecmp(value, "auto") || atobool(value) ? x264_cpu_detect() : 0;
         if( b_error )
         {
-            char *buf = strdup( value );
+            char *buf = x264_strdup( value );
             if( buf )
             {
                 char *tok, UNUSED *saveptr=NULL, *init;
@@ -632,7 +638,7 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
                     if( !x264_cpu_names[i].flags )
                         b_error = 1;
                 }
-                free( buf );
+                x264_free( buf );
                 if( (p->cpu&X264_CPU_SSSE3) && !(p->cpu&X264_CPU_SSE2_IS_SLOW) )
                     p->cpu |= X264_CPU_SSE2_IS_FAST;
             }
@@ -818,10 +824,10 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         else if( strstr( value, "jvt" ) )
             p->i_cqm_preset = X264_CQM_JVT;
         else
-            p->psz_cqm_file = strdup(value);
+            p->psz_cqm_file = x264_strdup(value);
     }
     OPT("cqmfile")
-        p->psz_cqm_file = strdup(value);
+        p->psz_cqm_file = x264_strdup(value);
     OPT("cqm4")
     {
         p->i_cqm_preset = X264_CQM_CUSTOM;
@@ -885,7 +891,7 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
     OPT("log")
         p->i_log_level = atoi(value);
     OPT("dump-yuv")
-        p->psz_dump_yuv = strdup(value);
+        p->psz_dump_yuv = x264_strdup(value);
     OPT2("analyse", "partitions")
     {
         p->analyse.inter = 0;
@@ -999,11 +1005,13 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         p->rc.b_stat_write = pass & 1;
         p->rc.b_stat_read = pass & 2;
     }
+#if HAVE_FILEIO
     OPT("stats")
     {
-        p->rc.psz_stat_in = strdup(value);
-        p->rc.psz_stat_out = strdup(value);
+        p->rc.psz_stat_in = x264_strdup(value);
+        p->rc.psz_stat_out = x264_strdup(value);
     }
+#endif
     OPT("qcomp")
         p->rc.f_qcompress = atof(value);
     OPT("mbtree")
@@ -1013,7 +1021,7 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
     OPT2("cplxblur", "cplx-blur")
         p->rc.f_complexity_blur = atof(value);
     OPT("zones")
-        p->rc.psz_zones = strdup(value);
+        p->rc.psz_zones = x264_strdup(value);
     OPT("crop-rect")
         b_error |= sscanf( value, "%u,%u,%u,%u", &p->crop_rect.i_left, &p->crop_rect.i_top,
                                                  &p->crop_rect.i_right, &p->crop_rect.i_bottom ) != 4;
@@ -1045,12 +1053,14 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         p->i_frame_packing = atoi(value);
     OPT("stitchable")
         p->b_stitchable = atobool(value);
+#if HAVE_OPENCL
     OPT("opencl")
         p->b_opencl = atobool( value );
     OPT("opencl-clbin")
-        p->psz_clbin_file = strdup( value );
+        p->psz_clbin_file = x264_strdup( value );
     OPT("opencl-device")
         p->i_opencl_device = atoi( value );
+#endif
     else
     {
         b_error = 1;
@@ -1063,12 +1073,13 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
 #undef atof
 
     if( name_buf )
-        free( name_buf );
+        x264_free( name_buf );
 
     b_error |= value_was_null && !name_was_bool;
     return b_error ? errortype : 0;
 }
 
+#if HAVE_FILEIO
 /****************************************************************************
  * x264_log:
  ****************************************************************************/
@@ -1110,6 +1121,7 @@ static void x264_log_default( void *p_unused, int i_level, const char *psz_fmt,
     fprintf( stderr, "x264 [%s]: ", psz_prefix );
     x264_vfprintf( stderr, psz_fmt, arg );
 }
+#endif
 
 /****************************************************************************
  * x264_picture_init:
@@ -1266,6 +1278,7 @@ void name( type *n, type *d )\
 REDUCE_FRACTION( x264_reduce_fraction  , uint32_t )
 REDUCE_FRACTION( x264_reduce_fraction64, uint64_t )
 
+#if HAVE_FILEIO
 /****************************************************************************
  * x264_slurp_file:
  ****************************************************************************/
@@ -1307,6 +1320,7 @@ error:
     fclose( fh );
     return NULL;
 }
+#endif
 
 /****************************************************************************
  * x264_param2string:
diff --git a/common/common.h b/common/common.h
index c7850ca..8bae13f 100644
--- a/common/common.h
+++ b/common/common.h
@@ -254,15 +254,29 @@ static const uint8_t x264_scan8[16*3 + 3] =
 void *x264_malloc( int );
 void  x264_free( void * );
 
+#if HAVE_FILEIO
 /* x264_slurp_file: malloc space for the whole file and read it */
 char *x264_slurp_file( const char *filename );
+#endif
 
 /* x264_param2string: return a (malloced) string containing most of
  * the encoding options */
 char *x264_param2string( x264_param_t *p, int b_res );
 
+/* copy s to new string variable located in heap memory */
+static ALWAYS_INLINE char *x264_strdup( const char *s )
+{
+    char *s2 = x264_malloc(strlen(s) + 1);
+    strcpy(s2,s);
+    return s2;
+}
+
 /* log */
+#if HAVE_FILEIO
 void x264_log( x264_t *h, int i_level, const char *psz_fmt, ... );
+#else
+#define x264_log(h,i_level,psz_fmt,...)
+#endif
 
 void x264_reduce_fraction( uint32_t *n, uint32_t *d );
 void x264_reduce_fraction64( uint64_t *n, uint64_t *d );
diff --git a/common/cpu.c b/common/cpu.c
index 636a40c..28c4011 100644
--- a/common/cpu.c
+++ b/common/cpu.c
@@ -96,7 +96,7 @@ const x264_cpu_name_t x264_cpu_names[] =
     {"", 0},
 };
 
-#if (ARCH_PPC && SYS_LINUX) || (ARCH_ARM && !HAVE_NEON)
+#if (ARCH_PPC && SYS_LINUX) || (ARCH_ARM && HAVE_ARMV6 && !HAVE_NEON)
 #include <signal.h>
 #include <setjmp.h>
 static sigjmp_buf jmpbuf;
diff --git a/common/osdep.h b/common/osdep.h
index ed3ed59..9043c58 100644
--- a/common/osdep.h
+++ b/common/osdep.h
@@ -103,6 +103,12 @@ int x264_is_pipe( const char *path );
 #define x264_is_pipe(x) 0
 #endif
 
+/* if x264 should not use default stdlib heap */
+#if !HAVE_HEAP
+#define malloc __x264_malloc
+#define free __x264_free
+#endif
+
 #ifdef _MSC_VER
 #define DECLARE_ALIGNED( var, n ) __declspec(align(n)) var
 #else
@@ -321,7 +327,7 @@ static ALWAYS_INLINE uint16_t endian_fix16( uint16_t x )
 /* For values with 4 bits or less. */
 static int ALWAYS_INLINE x264_ctz_4bit( uint32_t x )
 {
-    static uint8_t lut[16] = {4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0};
+    static const uint8_t lut[16] = {4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0};
     return lut[x];
 }
 
@@ -331,7 +337,7 @@ static int ALWAYS_INLINE x264_ctz_4bit( uint32_t x )
 #else
 static int ALWAYS_INLINE x264_clz( uint32_t x )
 {
-    static uint8_t lut[16] = {4,3,2,2,1,1,1,1,0,0,0,0,0,0,0,0};
+    static const uint8_t lut[16] = {4,3,2,2,1,1,1,1,0,0,0,0,0,0,0,0};
     int y, z = (((x >> 16) - 1) >> 27) & 16;
     x >>= z^16;
     z += y = ((x - 0x100) >> 28) & 8;
@@ -343,7 +349,7 @@ static int ALWAYS_INLINE x264_clz( uint32_t x )
 
 static int ALWAYS_INLINE x264_ctz( uint32_t x )
 {
-    static uint8_t lut[16] = {4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0};
+    static const uint8_t lut[16] = {4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0};
     int y, z = (((x & 0xffff) - 1) >> 27) & 16;
     x >>= z;
     z += y = (((x & 0xff) - 1) >> 28) & 8;
@@ -397,18 +403,26 @@ static ALWAYS_INLINE void x264_prefetch( void *p )
 
 static inline int x264_is_regular_file( FILE *filehandle )
 {
+#if HAVE_FILEIO
     x264_struct_stat file_stat;
     if( x264_fstat( fileno( filehandle ), &file_stat ) )
         return 1;
     return S_ISREG( file_stat.st_mode );
+#else
+    return 0;
+#endif
 }
 
 static inline int x264_is_regular_file_path( const char *filename )
 {
+#if HAVE_FILEIO
     x264_struct_stat file_stat;
     if( x264_stat( filename, &file_stat ) )
         return !x264_is_pipe( filename );
     return S_ISREG( file_stat.st_mode );
+#else
+    return 0;
+#endif
 }
 
 #endif /* X264_OSDEP_H */
diff --git a/common/set.c b/common/set.c
index b9d94fd..0e13e31 100644
--- a/common/set.c
+++ b/common/set.c
@@ -344,6 +344,7 @@ static int x264_cqm_parse_jmlist( x264_t *h, const char *buf, const char *name,
     return 0;
 }
 
+#if HAVE_FILEIO
 int x264_cqm_parse_file( x264_t *h, const char *filename )
 {
     char *p;
@@ -376,4 +377,4 @@ int x264_cqm_parse_file( x264_t *h, const char *filename )
     x264_free( buf );
     return b_error;
 }
-
+#endif
diff --git a/common/vlc.c b/common/vlc.c
index a55d4b9..8e1d08a 100644
--- a/common/vlc.c
+++ b/common/vlc.c
@@ -798,8 +798,8 @@ static const vlc_t run_before[7][16] =
     },
 };
 
-vlc_large_t x264_level_token[7][LEVEL_TABLE_SIZE];
-uint32_t x264_run_before[1<<16];
+vlc_large_t x264_level_token[7][LEVEL_TABLE_SIZE] X264_LARGE_BSS_LINK_SECTION;
+uint32_t x264_run_before[1<<16] X264_LARGE_BSS_LINK_SECTION;
 
 void x264_cavlc_init( x264_t *h )
 {
diff --git a/configure b/configure
index 293d008..cfb3c4c 100755
--- a/configure
+++ b/configure
@@ -28,6 +28,9 @@ Configuration options:
   --disable-opencl         disable OpenCL features
   --disable-gpl            disable GPL-only features
   --disable-thread         disable multithreaded encoding
+  --disable-fileio         disable stdio file support
+  --disable-heap           disable stdlib heap support
+  --disable-bss            disable common bss memory
   --disable-win32thread    disable win32threads (windows only)
   --disable-interlaced     disable interlaced encoding support
   --bit-depth=BIT_DEPTH    set output bit depth (8-10) [8]
@@ -349,6 +352,9 @@ asm="auto"
 interlaced="yes"
 lto="no"
 debug="no"
+fileio="auto"
+heap="auto"
+bss="auto"
 gprof="no"
 strip="no"
 pic="no"
@@ -376,7 +382,7 @@ NL="
 # list of all preprocessor HAVE values we can define
 CONFIG_HAVE="MALLOC_H ALTIVEC ALTIVEC_H MMX ARMV6 ARMV6T2 NEON BEOSTHREAD POSIXTHREAD WIN32THREAD THREAD LOG2F SWSCALE \
              LAVF FFMS GPAC AVS GPL VECTOREXT INTERLACED CPU_COUNT OPENCL THP LSMASH X86_INLINE_ASM AS_FUNC INTEL_DISPATCHER \
-             MSA MMAP WINRT VSX"
+             MSA MMAP WINRT VSX FILEIO HEAP BSS"
 
 # parse options
 
@@ -449,6 +455,15 @@ for opt do
         --disable-thread)
             thread="no"
             ;;
+        --disable-fileio)
+            fileio="no"
+            ;;
+        --disable-heap)
+            heap="no"
+            ;;
+        --disable-bss)
+            bss="no"
+            ;;
         --disable-win32thread)
             [ "$thread" != "no" ] && thread="posix"
             ;;
@@ -624,6 +639,10 @@ case $host_os in
         define HAVE_MALLOC_H
         libm="-lm"
         ;;
+    none)
+        SYS="NONE"
+        libm="-lm"
+        ;;
     cygwin*|mingw*|msys*)
         EXE=".exe"
         if [[ $host_os = cygwin* ]] && cpp_check "" "" "defined(__CYGWIN__)" ; then
@@ -1198,6 +1217,16 @@ if [ "$debug" != "yes" -a "$gprof" != "yes" ]; then
     CFLAGS="$CFLAGS -fomit-frame-pointer"
 fi
 
+[ "$fileio" != "no" ] && define HAVE_FILEIO
+
+[ "$heap" != "no" ] && define HAVE_HEAP
+
+if [ "$bss" != "no" ]; then
+    CFLAGS="$CFLAGS -DX264_LARGE_BSS_LINK_SECTION=\"\""
+else
+    CFLAGS="$CFLAGS -DX264_LARGE_BSS_LINK_SECTION=\"__attribute__ ((section(\\\".x264.bss\\\")))\""
+fi
+
 if [ "$strip" = "yes" ]; then
     LDFLAGS="$LDFLAGS -s"
 fi
@@ -1500,6 +1529,9 @@ opencl:        $opencl
 filters:       $filters
 lto:           $lto
 debug:         $debug
+fileio:        $fileio
+heap:          $heap
+bss:           $bss
 gprof:         $gprof
 strip:         $strip
 PIC:           $pic
diff --git a/encoder/analyse.c b/encoder/analyse.c
index 1941bf2..63652e4 100644
--- a/encoder/analyse.c
+++ b/encoder/analyse.c
@@ -272,9 +272,9 @@ static const uint8_t i_sub_mb_p_cost_table[4] =
 
 static void x264_analyse_update_cache( x264_t *h, x264_mb_analysis_t *a );
 
-static uint16_t x264_cost_ref[QP_MAX+1][3][33];
+static uint16_t x264_cost_ref[QP_MAX+1][3][33] X264_LARGE_BSS_LINK_SECTION;
 static UNUSED x264_pthread_mutex_t cost_ref_mutex = X264_PTHREAD_MUTEX_INITIALIZER;
-static uint16_t x264_cost_i4x4_mode[(QP_MAX+2)*32];
+static uint16_t x264_cost_i4x4_mode[(QP_MAX+2)*32] X264_LARGE_BSS_LINK_SECTION;
 
 static int init_costs( x264_t *h, float *logs, int qp )
 {
diff --git a/encoder/encoder.c b/encoder/encoder.c
index 27db1bd..cec739e 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -79,6 +79,7 @@ static int x264_threadpool_wait_all( x264_t *h )
     return 0;
 }
 
+#if HAVE_FILEIO
 static void x264_frame_dump( x264_t *h )
 {
     FILE *f = x264_fopen( h->param.psz_dump_yuv, "r+b" );
@@ -113,6 +114,7 @@ static void x264_frame_dump( x264_t *h )
     }
     fclose( f );
 }
+#endif
 
 /* Fill "default" values */
 static void x264_slice_header_init( x264_t *h, x264_slice_header_t *sh,
@@ -1433,14 +1435,16 @@ x264_t *x264_encoder_open( x264_param_t *param )
     if( x264_validate_parameters( h, 1 ) < 0 )
         goto fail;
 
+#if HAVE_FILEIO
     if( h->param.psz_cqm_file )
         if( x264_cqm_parse_file( h, h->param.psz_cqm_file ) < 0 )
             goto fail;
 
     if( h->param.rc.psz_stat_out )
-        h->param.rc.psz_stat_out = strdup( h->param.rc.psz_stat_out );
+        h->param.rc.psz_stat_out = x264_strdup( h->param.rc.psz_stat_out );
     if( h->param.rc.psz_stat_in )
-        h->param.rc.psz_stat_in = strdup( h->param.rc.psz_stat_in );
+        h->param.rc.psz_stat_in = x264_strdup( h->param.rc.psz_stat_in );
+#endif
 
     x264_reduce_fraction( &h->param.i_fps_num, &h->param.i_fps_den );
     x264_reduce_fraction( &h->param.i_timebase_num, &h->param.i_timebase_den );
@@ -1691,6 +1695,7 @@ x264_t *x264_encoder_open( x264_param_t *param )
         x264_log( h, X264_LOG_DEBUG, "CPB size: %i bits\n", h->sps->vui.hrd.i_cpb_size_unscaled );
     }
 
+#if HAVE_FILEIO
     if( h->param.psz_dump_yuv )
     {
         /* create or truncate the reconstructed video file */
@@ -1708,6 +1713,7 @@ x264_t *x264_encoder_open( x264_param_t *param )
         }
         fclose( f );
     }
+#endif
 
     const char *profile = h->sps->i_profile_idc == PROFILE_BASELINE ? "Constrained Baseline" :
                           h->sps->i_profile_idc == PROFILE_MAIN ? "Main" :
@@ -3280,8 +3286,10 @@ int     x264_encoder_encode( x264_t *h,
 
         if( h->param.rc.b_mb_tree && h->param.rc.b_stat_read )
         {
+#if HAVE_FILEIO
             if( x264_macroblock_tree_read( h, fenc, pic_in->prop.quant_offsets ) )
                 return -1;
+#endif
         }
         else
             x264_stack_align( x264_adaptive_quant_frame, h, fenc, pic_in->prop.quant_offsets );
@@ -4005,8 +4013,10 @@ static int x264_encoder_frame_end( x264_t *h, x264_t *thread_current,
             h->fref[0][i] = 0;
         }
 
+#if HAVE_FILEIO
     if( h->param.psz_dump_yuv )
         x264_frame_dump( h );
+#endif
     x264_emms();
 
     return frame_size;
@@ -4330,11 +4340,13 @@ void    x264_encoder_close  ( x264_t *h )
     /* rc */
     x264_ratecontrol_delete( h );
 
+#if HAVE_FILEIO
     /* param */
     if( h->param.rc.psz_stat_out )
-        free( h->param.rc.psz_stat_out );
+        x264_free( h->param.rc.psz_stat_out );
     if( h->param.rc.psz_stat_in )
-        free( h->param.rc.psz_stat_in );
+        x264_free( h->param.rc.psz_stat_in );
+#endif
 
     x264_cqm_delete( h );
     x264_free( h->nal_buffer );
diff --git a/encoder/ratecontrol.c b/encoder/ratecontrol.c
index dbccb27..a26dad5 100644
--- a/encoder/ratecontrol.c
+++ b/encoder/ratecontrol.c
@@ -117,6 +117,7 @@ struct x264_ratecontrol_t
     double ip_offset;
     double pb_offset;
 
+#if HAVE_FILEIO
     /* 2pass stuff */
     FILE *p_stat_file_out;
     char *psz_stat_file_tmpname;
@@ -124,6 +125,7 @@ struct x264_ratecontrol_t
     char *psz_mbtree_stat_file_tmpname;
     char *psz_mbtree_stat_file_name;
     FILE *p_mbtree_stat_file_in;
+#endif
 
     int num_entries;            /* number of ratecontrol_entry_ts */
     ratecontrol_entry_t *entry; /* FIXME: copy needed data and free this once init is done */
@@ -505,6 +507,7 @@ static ALWAYS_INLINE float tapfilter( float *src, int pos, int max, int stride,
     return sum;
 }
 
+#if HAVE_FILEIO
 static void x264_macroblock_tree_rescale( x264_t *h, x264_ratecontrol_t *rc, float *dst )
 {
     float *input, *output;
@@ -580,6 +583,7 @@ fail:
     x264_log( h, X264_LOG_ERROR, "Incomplete MB-tree stats file.\n" );
     return -1;
 }
+#endif
 
 int x264_reference_build_list_optimal( x264_t *h )
 {
@@ -618,6 +622,7 @@ int x264_reference_build_list_optimal( x264_t *h )
     return 0;
 }
 
+#if HAVE_FILEIO
 static char *x264_strcat_filename( char *input, char *suffix )
 {
     char *output = x264_malloc( strlen( input ) + strlen( suffix ) + 1 );
@@ -627,6 +632,7 @@ static char *x264_strcat_filename( char *input, char *suffix )
     strcat( output, suffix );
     return output;
 }
+#endif
 
 void x264_ratecontrol_init_reconfigurable( x264_t *h, int b_init )
 {
@@ -863,6 +869,7 @@ int x264_ratecontrol_new( x264_t *h )
         return -1;
     }
 
+#if HAVE_FILEIO
     /* Load stat file and init 2pass algo */
     if( h->param.rc.b_stat_read )
     {
@@ -1184,6 +1191,7 @@ parse_error:
             }
         }
     }
+#endif
 
     if( h->param.rc.b_mb_tree && (h->param.rc.b_stat_read || h->param.rc.b_stat_write) )
     {
@@ -1352,6 +1360,7 @@ void x264_ratecontrol_summary( x264_t *h )
 void x264_ratecontrol_delete( x264_t *h )
 {
     x264_ratecontrol_t *rc = h->rc;
+#if HAVE_FILEIO
     int b_regular_file;
 
     if( rc->p_stat_file_out )
@@ -1381,6 +1390,7 @@ void x264_ratecontrol_delete( x264_t *h )
     }
     if( rc->p_mbtree_stat_file_in )
         fclose( rc->p_mbtree_stat_file_in );
+#endif
     x264_free( rc->pred );
     x264_free( rc->pred_b_from_p );
     x264_free( rc->entry );
@@ -1829,6 +1839,7 @@ int x264_ratecontrol_end( x264_t *h, int bits, int *filler )
     h->fdec->f_qp_avg_aq = (float)rc->qpa_aq / h->mb.i_mb_count;
     h->fdec->f_crf_avg = h->param.rc.f_rf_constant + h->fdec->f_qp_avg_rc - rc->qp_novbv;
 
+#if HAVE_FILEIO
     if( h->param.rc.b_stat_write )
     {
         char c_type = h->sh.i_type==SLICE_TYPE_I ? (h->fenc->i_poc==0 ? 'I' : 'i')
@@ -1897,6 +1908,7 @@ int x264_ratecontrol_end( x264_t *h, int bits, int *filler )
                 goto fail;
         }
     }
+#endif
 
     if( rc->b_abr )
     {
diff --git a/encoder/rdo.c b/encoder/rdo.c
index cd76682..7a2ab1d 100644
--- a/encoder/rdo.c
+++ b/encoder/rdo.c
@@ -32,8 +32,8 @@
 
 /* Transition and size tables for abs<9 MVD and residual coding */
 /* Consist of i_prefix-2 1s, one zero, and a bypass sign bit */
-uint8_t x264_cabac_transition_unary[15][128];
-uint16_t x264_cabac_size_unary[15][128];
+uint8_t x264_cabac_transition_unary[15][128] X264_LARGE_BSS_LINK_SECTION;
+uint16_t x264_cabac_size_unary[15][128] X264_LARGE_BSS_LINK_SECTION;
 /* Transition and size tables for abs>9 MVD */
 /* Consist of 5 1s and a bypass sign bit */
 static uint8_t cabac_transition_5ones[128];
diff --git a/x264.c b/x264.c
index 41f94e4..1f92c81 100644
--- a/x264.c
+++ b/x264.c
@@ -1626,18 +1626,18 @@ generic_option:
     else FAIL_IF_ERROR( !info.vfr && input_opt.timebase, "--timebase is incompatible with cfr input\n" );
 
     /* init threaded input while the information about the input video is unaltered by filtering */
-#if HAVE_THREAD
     if( info.thread_safe && (b_thread_input || param->i_threads > 1
         || (param->i_threads == X264_THREADS_AUTO && x264_cpu_num_processors() > 1)) )
     {
+#if HAVE_THREAD
         if( thread_input.open_file( NULL, &opt->hin, &info, NULL ) )
         {
             fprintf( stderr, "x264 [error]: threaded input failed\n" );
             return -1;
         }
         cli_input = thread_input;
-    }
 #endif
+    }
 
     /* override detected values by those specified by the user */
     if( param->vui.i_sar_width > 0 && param->vui.i_sar_height > 0 )
